/*	ClientToServerHandler.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil;

import profil.http.Packet;
import profil.filter.FilterEngine;
import java.net.*;
import java.io.*;


/**
 * <p>
 * Handles all data going from the client to the server, and filters it.
 * <code>ClientToServerHandler</code> is created by the <code>ProxyServer</code>
 * each time a new connection to ProFil is made by a user agent. This
 * <code>ClientToServerHandler</code> is in charge of reading requests coming from
 * the user agent, selecting the <code>SiteFilter</code> associated with these
 * requests, and filter them. If after filtering, the connection to the server
 * is allowed, <code>ClientToServerHandler</code> creates a <code>ServerToClientHandler</code>
 * that handles the replies sent by the server. Otherwise, it sends a reply message
 * directly to the client (usually a '404 not found').
 * Then the <code>ClientToServerHandler</code> just loops, reading data from the client,
 * filtering it, then eventually sending it to the server.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public class ClientToServerHandler extends Thread
{
	private SocketHandler sh;
	private SiteFilterStore store;
	
	/**
	 * Creates a new <code>ClientToServerHandler</code>. It needs a
	 * <code>SocketHandler</code> providing read and write access to the
	 * client's socket, and a <code>SiteFilterStore</code> from which
	 * to extract the <code>SiteFilter</code> corresponding to the
	 * connection handled.
	 * <p>
	 * @param <code>sh</code> the <code>SocketHandler</code> for this connection.
	 * @param <code>store</code> ProFil's <code>SiteFilterStore</code>.
	 */
	public ClientToServerHandler(SocketHandler sh, SiteFilterStore store)
	{	this.store 	= store;
		this.sh 	= sh;
	}
	
	
	/**
	 * Start this <code>ClientToServerHandler</code>.
	 */
	public void run()
	{		
		try {	Packet packet = new Packet(sh.getProxy());
				
				while(true)
				{
					//wait for a 1st packet and fetch it
					while (!packet.headerReceived())
					{	sh.readFromClient(packet);	}
String rq = (packet.getRequest()).toString();
rq = rq.substring(0, rq.length()-4);		
System.out.println("\nCtS> "+rq);

					//select apropriate filter for this connection
					FilterEngine filters = store.getFilterEngineFor(packet);
					filters.filter(packet);


					if (!packet.canBeSent())
					{	replyNotFound();	
						sh.close();
						return;
					}
					else
					{	//connect to server
						sh.connect((packet.getRequest()).getServer());

						//build ServerHandler
						(new ServerToClientHandler(sh, filters)).start();
						sh.writeToServer(packet);

						while (packet.waitingContent())
						{
							sh.readFromClient(packet);
							filters.filter(packet);
							sh.writeToServer(packet);
						}
					}
				}
				
		} catch (Exception e)	//ie client has unfriendly closed connection
		{		}				//or unable to build ServerToClientHandler
	}
		
	//sends a 404 back to the client
	private void replyNotFound()	throws IOException
	{	String message = "HTTP/1.0 404 Not allowed by ProFil\n\n";
		sh.writeToClient(message);
	}

}