/*	ProFilGUI.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil;

import profil.http.Proxy;
import java.awt.*;
import java.awt.event.*;
import java.util.zip.*;
import java.io.*;


/**
 * <p>
 * ProFil's main Graphical User Interface.
 * The User can use it to configure ProFil's listening port, to make ProFil
 * use an intermediary proxy, and to manage the <code>SiteFilter</code>s.
 * <code>SiteFilter</code>s can be created (by opening a <code>SiteFilterGUI</code>s),
 * deleted, edited, or activated/desactivated. This last feature allows to temporarily
 * stop using a filter, without having to delete it. Tests are also run to be sure the User
 * enters proper values.
 * <p>
 * <code>SiteFilter</code>s are mainly defined by a Perl5 regular expression, a priority, and
 * a set of actions (the filters). Each time ProFil handles a request from an HTTP client to
 * a server, it extracts the requiered URL from it, and try to match it with one 
 * of the <code>SiteFilter</code>s' regular expressions. When one match is found, the filters
 * of this <code>SiteFilter</code> will be applied to any data going way or back on this
 * connection. Be careful that ProFil only considers the first <code>SiteFilter</code>
 * whose regular expression matches the URL. Thus, overlapping <code>SiteFilter</code>s
 * (different <code>SiteFilter</code>s whose regular expressions could match the same URL)
 * have to be avoided. Perl5 regular expressions are thought to be acurate enough to
 * do this. Finaly, when no match was found, ProFil uses the filters of the
 * <code>default SiteFilter</code>.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */
 

public class ProFilGUI extends Frame implements ActionListener, WindowListener
{
		
	// Profil's variables
	private static String filename = "profil.cfg";
	private static SiteFilterStore store;
	private static Proxy proxy;
	private static ProxyServer server;
	private static int server_port;
	
	// GUI variables
	private TextField tf_port	= new TextField("", 5);
	private TextField tf_proxy	= new TextField("", 25);
	private Checkbox cb_proxy	= new Checkbox("Use Proxy");
	private Button b_default	= new Button("Default SiteFilter");
	private Button b_run		= new Button("Run ProFil");
	private Button b_new		= new Button("New");
	private Button b_edit		= new Button("Edit");
	private Button b_del		= new Button("Remove");
	private Button b_switch		= new Button("On/Off");
	private List l_list			= new List(10);
	private Label l_port		= new Label("Server run on port");
	private Label l_info		= new Label("");
	
	
	

	
	
	/**
	 * Displays ProFil's main GUI.
	 * It first tries to load ProFil's configuration (saved in file <code>"profil.cfg"</code>).
	 * In case of failure, a default config is load instead. Then, each time ProFil's server
	 * is started, stopped or the window is closed, the configuration is saved anew.
	 * This configuration contains all datas about the <code>SiteFilter</code>s and ProFil's
	 * proxy and running port.
	 */
	
	public ProFilGUI()
	{
		super("ProFil - HTTP Proxy");
		
		loadConfig();
		
		tf_port.setText(""+server_port);
		tf_proxy.setText(proxy.toString());
		cb_proxy.setState(proxy.isEnabled());
		
		//initialise List of store
		refreshList();
		
		//build GUI
		this.setBackground(new Color(192, 192, 192));
		this.addWindowListener(this);

		b_run.addActionListener(this);
		b_new.addActionListener(this);
		b_edit.addActionListener(this);
		b_del.addActionListener(this);
		b_switch.addActionListener(this);
		b_default.addActionListener(this);
		
		Panel north = new Panel();
		north.setLayout(new GridLayout(4,1));
		
		Panel p1 = new Panel();
		p1.setLayout(new FlowLayout(0,1,1));
		p1.add(l_port);
		p1.add(tf_port);
		north.add(p1);
		
		Panel p2 = new Panel();
		p2.setLayout(new FlowLayout(0,1,1));
		p2.add(cb_proxy);
		p2.add(tf_proxy);
		north.add(p2);
		
		north.add(b_run);
		north.add(l_info);
		
		Panel west = new Panel();
		west.setLayout(new GridLayout(4, 1, 5, 5));
		west.add(b_new);
		west.add(b_edit);
		west.add(b_del);
		west.add(b_switch);
		
		Panel center = new Panel();
		center.add("Center", l_list);
		
		Panel east = new Panel();
		east.setLayout(new GridLayout(4,1));
		Panel pp2 = new Panel();
		pp2.add(b_default);
		east.add(pp2);
		east.add(new Label(""));
		east.add(new Label(""));
		east.add(new Label(""));
		
		
		this.add("North", north);
		this.add("West", west);
		this.add("Center", center);
		this.add("East", east);
		
		this.pack();
		this.show();
	}
	
	/* Displays a text in the GUI.
	 * Used by <code>ProxyServer</code> to display messages in a blank
	 * Label of the GUI, just under the "Start Server" button.
	 * <p>
	 * @param <code>msg</code> the message to display.
	 */
	public void setInfo(String msg)
	{	l_info.setText(msg);		}
	
	
	
	
	/* WINDOW LISTENER
	 *	allows users to stop the program when closing the window
	 */

	public void windowClosing(WindowEvent e)		{	saveConfig();
														System.exit(0);	}
	public void windowClosed(WindowEvent e) 		{	}
	public void windowOpened(WindowEvent e) 		{	}
	public void windowIconified(WindowEvent e) 		{	}
	public void windowDeiconified(WindowEvent e) 	{	}
	public void windowActivated(WindowEvent e) 		{	}
	public void windowDeactivated(WindowEvent e) 	{	}



	/* ACTION LISTENER
	 */

	public void actionPerformed(ActionEvent e)
	{	String message = e.getActionCommand();
	
		if (message.equals("Run ProFil"))			do_run();
		if (message.equals("Stop ProFil"))			do_stop();
		if (message.equals("New"))					do_new();
		if (message.equals("Edit"))					do_edit();
		if (message.equals("Remove"))				do_remove();
		if (message.equals("On/Off"))				do_switch();
		if (message.equals("Default SiteFilter"))	do_default();
	}
	
	
	private void do_run()
	{	
		//various tests to check the validity of the proxy's address:port
		String address, p=tf_proxy.getText();
		int port = 0;
		
		if (p.trim().equals("") && cb_proxy.getState())
		{	new MessageGUI(this, "If you want to use a proxy, you have to enter its address:port");
			return;
		}
		
		if (!p.trim().equals(""))
		{
			int i = p.indexOf(":");
			if (i == -1)
			{	new MessageGUI(this, "The proxy's address and port need to be formated like: 'address:port'");
					return;
			}
			
			address = p.substring(0, i);
			if (address.trim().equals(""))
			{	new MessageGUI(this, "You need to specify an address to the proxy");
				return;
			}
		
			try {	port = Integer.parseInt(p.substring(i+1));	}
			catch(Exception e)
			{	new MessageGUI(this, "The proxy's port must be a number");
				return;
			}
		
			proxy = new Proxy(address, port, cb_proxy.getState());
		}
		
		//now check and set server's port
		try {	server_port = Integer.parseInt(tf_port.getText());	}
		catch (Exception e)
		{	new MessageGUI(this, "The server's port need to be a number");
			return;
		}
		
		//everything ok -> start server, update GUI
		saveConfig();
		
		b_run.setLabel("Stop ProFil");
	
		((Component)l_port).setEnabled(false);
		((Component)cb_proxy).setEnabled(false);
		((Component)tf_port).setEnabled(false);
		((Component)tf_proxy).setEnabled(false);
		
		server = new ProxyServer(proxy, server_port, store, this);
		server.start();
	}
	
	private void do_stop()
	{	saveConfig();
		server.stopServer();	
		b_run.setLabel("Run ProFil");
	
		((Component)l_port).setEnabled(true);
		((Component)cb_proxy).setEnabled(true);
		((Component)tf_port).setEnabled(true);
		((Component)tf_proxy).setEnabled(true);
	}
	
	private void do_default()
	{	SiteFilter f = store.getDefault();
		new SiteFilterGUI(store, f, this);
	}
	
	private void do_new()
	{	new SiteFilterGUI(store, this);
		refreshList();
	}
	
	private void do_edit()
	{	String item = getSiteFilterName();
		if (item.equals(""))	return;
		
		SiteFilter site = store.getSiteFilter(item);
		new SiteFilterGUI(store, site, this);
		refreshList();
	}
	
	private void do_remove()
	{	String item = getSiteFilterName();
		if (item.equals(""))	return;
		
		store.delSiteFilter(item);
		refreshList();
	}
	
	private void do_switch()
	{	String item = getSiteFilterName();
		if (item.equals(""))	return;
		
		store.switchSiteFilter(item);
		refreshList();
	}
	
	
	/////////////     PRIVATES     ///////////////////
	
	private void refreshList()
	{	store.refreshList(l_list);	}
	
	private String getSiteFilterName()
	{	String name = l_list.getSelectedItem();
		try {	name = name.substring(name.lastIndexOf(" ")+1);	}
		catch (Exception e)
		{	new MessageGUI(this, "You must select a SiteFilter first");
			return "";
		}
		return name;
	}
	
	
	//////////// LOAD & SAVE config ///////////////////
	
	private static void loadConfig()
	{		
		System.out.println("Loading Proxy & SiteFilters");
		try {	FileInputStream fin = new FileInputStream(filename);
				GZIPInputStream gzin = new GZIPInputStream(fin);
				ObjectInputStream in = new ObjectInputStream(gzin);
				store = (SiteFilterStore)in.readObject();
				proxy = (Proxy)in.readObject();
				server_port = in.readInt();
				in.close();
		}
		catch (Exception e)
		{	System.out.println("Exception while reading: "+e);
			store = new SiteFilterStore();
			proxy = new Proxy("", 0, false);
			server_port = 8080;
		}
	}
	
	private void saveConfig()
	{
		System.out.println("Saving Proxy & SiteFilters");
		try {	FileOutputStream fout = new FileOutputStream(filename);
				GZIPOutputStream gzout = new GZIPOutputStream(fout);
				ObjectOutputStream out = new ObjectOutputStream(gzout);
				out.writeObject(store);
				out.writeObject(proxy);
				out.writeInt(server_port);
				out.flush();
				out.close();
		}
		catch (IOException e) { System.out.println("Error saving Config: "+e); }
	}
}
