/*	ProxyServer.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

 
package profil;

import profil.http.Proxy;
import java.net.*;
import java.io.*;


/**
 * <p>
 * ProFil's server. <code>ProxyServer</code> runs as a <code>Thread</code>
 * and listens for incoming connections. It creates a <code>ClientToServerHandler</code>
 * for each of them, in charge of handling the data stream
 * going from a client to its target server.
 * This <code>ClientToServerHandler</code> will later and if required create
 * a <code>ServerToClientHandler</code> to handle the data stream on the way from the
 * target server to the client.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */
 
 
public class ProxyServer extends Thread
{
	// VARIABLES
	private static Proxy 			proxy;
	private static SiteFilterStore 	store;
	private static ProFilGUI 		gui;
	private static int 				port;
	private static boolean 			stop;
	private static int				count;	//amount of connections handled.
	
	
	/**
	 * Creates a new <code>ProxyServer</code>. It needs to be given the
	 * <code>port</code> on which to listen, as well as the <code>Proxy</code>
	 * used by ProFil and the <code>SiteFilterStore</code> to pass them
	 * to the <code>ClientToServerHandler</code>s. <code>ProxyServer</code>
	 * use a pointer to the main Graphical User Interface in order to display
	 * messages in it while running, such as error messages or the number of
	 * connections handled.
	 * <p>
	 * @param <code>proxy</code> ProFil's intermediary proxy, wether it is used or not.
	 * @param <code>port</code> the port on which ProFil will listen.
	 * @param <code>store</code> ProFil's <code>SiteFilterStore</code>.
	 * @param <code>gui</code> ProFil's main User Interface.
	 */
	public ProxyServer(Proxy proxy, int port, SiteFilterStore store, ProFilGUI gui)
	{
		this.proxy = proxy;
		this.port = port;
		this.store = store;
		this.gui = gui;
		this.stop = false;
		this.count = 0;
	}
	
	
	/**
	 * Stops the server. <code>stopServer()</code> stops the <code>ProxyServer</code>
	 * in a clean way by sending to its main thread a signal ordering it to stop
	 * accepting connections and to properly close every opened stream.
	 */
	
	public void stopServer()
	{	stop = true;	}
	
	
	/**
	 * Starts the server. If the server can't run on the given port, or if any error
	 * occurs, it will be displayed in the <code>ProFilGui</code> through its method
	 * <code>setInfo()</code>.
	 */
	public void run()
	{			
		try {
				ServerSocket server = new ServerSocket(port);
				server.setSoTimeout(1000);
				gui.setInfo("Server listening on port "+port);
		
				/***** wait for connection and create ConnectionHandler threads ******/
				while (true)
				{
					try {	Socket sock = server.accept();
							count++;
							gui.setInfo(""+count+" connection handled");
							SocketHandler sh = new SocketHandler(sock, proxy);
							(new ClientToServerHandler(sh, store)).start();
					}
					catch (InterruptedIOException e)
					{	if (stop)
						{	server.close();
							gui.setInfo("Server stopped.");
							return;	}
					}
					catch (Exception e)
					{	gui.setInfo("Server error:"+e.getMessage());
						return;
					}
				}
		
		} catch (Exception e)
		{	gui.setInfo("Server can't run on this port: "+port);		}
	}
}

