/*	SiteFilter.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil;

import com.oroinc.text.regex.*;
import profil.filter.Filter;
import profil.filter.FilterManager;
import profil.http.Packet;
import java.io.*;
import java.util.Vector;


/**
 * <p>
 * Defines a set of URIs to match and the filters to apply on them.
 * A <code>SiteFilter</code> is mainly defined by a Perl5 regular expression
 * used to match URIs of required documents, and a set of filters that will be applied
 * on any connection whose URI matches the <code>SiteFilter</code>'s
 * regular expression. This set of filters is inherited from <code>FilterEngine</code>.
 * <code>SiteFilter</code>s also have a name, and can be activated or desactivated.
 * <p>
 * To allow Perl5 pattern matching, ProFil uses the free package OROMatcher TM, the
 * best available package for this purpose.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public class SiteFilter extends profil.filter.FilterEngine implements Serializable
{
	private String regex;
	private String name;
	private boolean activ;
	private Pattern pattern;
	
	
	//private int priority;
	//public int getPriority()			{	return priority;	}
	//public void setPriority(int p)	{	this.priority = p;	}
	
	
	
	//overrides Serializable to take in charge Pattern
	private void writeObject(ObjectOutputStream out) throws IOException
	{
		super.save(out);
		out.writeUTF(regex);
		out.writeUTF(name);
		//out.writeInt(priority);
		out.writeBoolean(activ);
		out.writeObject(pattern);
		out.flush();	
	}

	//overrides Serializable to take in charge Pattern
	private void readObject(ObjectInputStream in) throws IOException, ClassNotFoundException
	{
		super.load(in);
		regex = in.readUTF();
		name = in.readUTF();
		//priority = in.readInt();
		activ = in.readBoolean();
		pattern = (Pattern)in.readObject();
	}
	
	
	
	/**
	 * Creates an empty <code>SiteFilter</code>. It has no name, no regular expression
	 * and no filters set. It is activated by default.
	 */
	public SiteFilter()
	{	super();
		regex = "";
		name = "";				//priority = 10;
		activ = true;
	}
	
	
	/**
	 * The <code>SiteFilter</code>'s regular expression as a String.
	 * <p>
	 * @return The <code>SiteFilter</code>'s regular expression.
	 */
	public String getRegex()			{	return regex;		}


	/**
	 * The <code>SiteFilter</code>'s name.
	 * <p>
	 * @return The <code>SiteFilter</code>'s name.
	 */
	public String getName()				{	return name;		}


	/**
	 * Test if the <code>SiteFilter</code> is activated.
	 * <p>
	 * @return <code>true</code> if activated, <code>false</code> otherwise.
	 */
	public boolean isActiv()			{	return activ;		}


	/**
	 * The <code>SiteFilter</code>'s regular expression as a <code>Pattern</code>.
	 * <code>Pattern</code> is defined in OROMatcher TM.
	 * <p>
	 * @return <code></code>
	 */
	public Pattern getPattern()			{	return pattern;		}


	/**
	 * Set this <code>SiteFilter</code>'s name.
	 * <p>
	 * @param <code>name</code> the new <code>SiteFilter</code>'s name.
	 */
	public void setName(String name)	{	this.name = name;	}


	/**
	 * Activates/Desactivates this <code>SiteFilter</code>.
	 * <p>
	 * @param <code>b</code> <code>true</code> to activate, <code>false</code> to desactivate. 
	 */
	public void setActiv(boolean b)		{	this.activ = b;		}
	

	/**
	 * Set this <code>SiteFilter</code>'s regular expression.
	 * Throws an exception if this regular expression is not correctly
	 * formated.
	 * <p>
	 * @param <code>regex</code> the new <code>SiteFilter</code>'s regular expression.
	 */
	public void setRegex(String regex) throws Exception
	{	this.regex = regex;
		Perl5Compiler perlc = new Perl5Compiler();
		this.pattern = perlc.compile(regex, Perl5Compiler.SINGLELINE_MASK | Perl5Compiler.CASE_INSENSITIVE_MASK);
	}
}
	
	