/*	SocketHandler.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil;

import profil.http.Packet;
import profil.http.Proxy;
import java.io.*;
import java.net.*;


/**
 * <p>
 * <code>SocketHandler</code> handles all the socket streams involved in a
 * client-server connection using ProFil. It provides methods to read or
 * write data from socket streams between ProFil and the server or the client
 * and to close these streams.
 * <p>
 * For each connection handled by ProFil, a <code>ClientToServerHandler</code> is created, that
 * eventually creates a <code>ServerToClientHandler</code> (if the need to connect to the server).
 * Both of them then share a common <code>SocketHandler</code> that manages all
 * the streams related to this connection.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public class SocketHandler
{
	private String enc = "ISO8859-1";	//latin-1
	
	private Socket client, server;
	
	private InputStream client_in, server_in;
	private OutputStream client_out, server_out;
	
	private Proxy proxy;
	
	private boolean client_closed;
	private boolean server_closed;
	
	
	/**
	 * Creates a new SocketHandler. It is already connected to the client
	 * through the <code>Socket client</code>. It is not yet connected to the
	 * server, and will be only if the <code>connect()</code> method is called.
	 * Then it might use the <code>Proxy</code>, which is why it is required
	 * in the constructor.
	 * <p>
	 * @param <code>client</code> the <code>Socket</code> connected to the user agent.
	 * @param <code>proxy</code> ProFil's proxy.
	 * @return a new <code>SocketHandler</code>.
	 */
	public SocketHandler(Socket client, Proxy proxy) throws IOException
	{
		this.client		= client;
		this.proxy 		= proxy;
		this.client_in 	= client.getInputStream();
		this.client_out = client.getOutputStream();
		
		this.client_closed = false;
		this.server_closed = false;
	}
	
	
	/**
	 * Connect this <code>SocketHandler</code> to the server <code>address</code>.
	 * If ProFil is using a proxy, the connection will be made with this proxy
	 * instead of the server itself.
	 * <p>
	 * @param <code>address</code> a String representing the address of the server to connect to.
	 */
	public void connect(String address) throws Exception
	{
		server 		= proxy.getServerSocket(address);
		server_in 	= server.getInputStream();
		server_out 	= server.getOutputStream();
	}
	
	
	/**
	 * Reads some data from a client and stores it in a <code>Packet</code>.
	 * <p>
	 * @param <code>packet</code> the packet in which to store the data read.
	 */
	public void readFromClient(Packet packet) throws IOException
	{	if (client_closed)	throw new IOException();
	
		try {	read(packet, client_in);	}
		catch (IOException e)
		{	
			client_closed = true;
			try {	
					client_in.close();
					client_out.close();
					client.close();
			} catch (Exception ee)	{}
		}
	}


	/**
	 * Reads some data from a server and stores it in a <code>Packet</code>.
	 * <p>
	 * @param <code>packet</code> the packet in which to store the data read.
	 */
	public void readFromServer(Packet packet) throws IOException
	{	if (server_closed)	throw new IOException();
	
		try {	read(packet, server_in);	}
		catch (IOException e)
		{
			server_closed = true;
			client_closed = true;
			close();
		}
	}		
	
	
	/**
	 * Sends to the client the data available in <code>packet</code>.
	 * <p>
	 * @param <code>packet</code> the packet whose data will be sent.
	 */
	public void writeToClient(Packet packet) throws IOException
	{	write(packet, client_out);	}
	
	
	/**
	 * Sends to the server the data available in <code>packet</code>.
	 * <p>
	 * @param <code>packet</code> the packet whose data will be sent.
	 */
	public void writeToServer(Packet packet) throws IOException
	{	write(packet, server_out);	}


	/**
	 * Sends a string to to the client. Does not require to be connected to
	 * the server.
	 * <p>
	 * @param <code>message</code> the string to send to the client.
	 */
	public void writeToClient(String message) throws IOException
	{
		synchronized(client_out)
		{	client_out.write(message.getBytes());
			client_out.flush();
		}
	}
		
		
	
	/**
	 * Closes all the streams and sockets used by this <code>SocketHandler</code>.
	 */
	public void close()
	{	
		try {	client_in.close();		//need to close the client side first
				client_out.close();		//since if server not connected, mistake
				client.close();			//when closing server side
				server_in.close();		// => what's following won't be executed
				server_out.close();
				server.close();
		} catch(Exception e)	{}
	}
	
	
	//read data from the stream in and store it in the packet
	private void read(Packet packet, InputStream in) throws IOException
	{	byte[] b = new byte[10000];		//by default, can read up to 10kbytes
		int i = in.read(b);

		if (i == -1)	throw new IOException();

		String st = new String(b, 0, i, enc);
		
		packet.add(st);
	}

	
	//take all data available for sending in packet and send it on stream out 
	private void write(Packet packet, OutputStream out) throws IOException
	{	
		String output = packet.get();	
		
		if (output.length() > 0)
		{			
			synchronized(out)
			{	out.write(output.getBytes(enc));
				out.flush();
			}
		}
	}
	

	/**
	 * Returns the proxy used by ProFil.
	 * <p>
	 * @return ProFil's proxy.
	 */
	public Proxy getProxy()		{	return proxy;	}
		
}
	
	