/*	Filter.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/


package profil.filter;

import profil.http.*;
import java.awt.Dialog;
import java.io.Serializable;


/**
 * <p>
 * The superclass of all filters used by ProFil.
 * <p>
 * ProFil is an HTTP proxy able to filter connections depending on their
 * target HTTP server. ProFil manages for this a set of <code>SiteFilter</code>
 * stored in a <code>SiteFilterStore<code>, and each of these <code>SiteFilter</code>
 * is defined by a Perl5 regular expression which has to match one or more URL query, and
 * by a set of 'filters'.
 * In this context, a 'filter' represents an 'action' that can be applied to the datas
 * going between HTTP servers and clients.
 * <p>
 * The class <code>Filter<code> is the superclass of all these 'filters', requiring of
 * them to implement a minimal set of functions: <code>getTYPE()<code>,
 * <code>filter()<code>, <code>edit()<code> and <code>duplicate()<code>.
 * <p>
 * Every new filter that needs to be added to ProFil has to extend class <code>Filter<code>.
 * The other steps of creating a new filter are related with the class
 * <code>FilterManager<code> and <code>FilterEngine</code> which need to be updated to
 * take the new filter into account. See <code>FilterManager<code> and
 * <code>FilterEngine</code> for more details.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public abstract class Filter implements Serializable
{
	/**
	 * Returns the filter's type. Each subclass of <code>Filter<code> possesses
	 * its own type number, as defined in class <code>FilterManager<code>. When
	 * <code>getTYPE()<code> is called on a <code>Filter<code> subclass's instance, it needs
	 * to return the number identifying the subclass of <code>Filter<code> to which
	 * it belongs.
	 * <p>
	 * @return an int specifying the <code>Filter<code>'s subclass of this <code>Filter<code>.
	 */
	public abstract int getTYPE();
	
	
	/**
	 * Filters the <code>Packet<code>. Each <code>Filter<code>'s subclass filters a specified
	 * part of the data going between a client and a server. Some will modify the data stream
	 * to forbid to load pictures, others to remove scripts from .html documents, etc...
	 * <p>
	 * @param <code>packet</code> the <code>Packet<code> to filter.
	 */
	public abstract void filter(Packet packet);
	
	/**
	 * Edit this <code>Filter<code>'s setting. For <code>Filter<code>'s subclasses
	 * that do not need any initialisation, <code>edit()<code> can just display a window
	 * (with <code>owner<code> as its owner window) explaining the action of this filter.
	 * Subclasses that require initialisation will use this method to display a graphical
	 * configuration interface.
	 * <p>
	 * @param <code>owner</code> the owner <code>Dialog<code> of the window to build. Typicaly the
	 * <code>Dialog<code> that called <code>edit()<code> on this <code>Filter<code>.
	 */
	public abstract void edit(Dialog owner);

	/**
	 * Returns a copy of this <code>Filter</code>. This method is required to allow
	 * to initiate or pass some variables specific to a <code>Filter<code>'s subclass.
	 * <p>
	 * @return a copy of this <code>Filter</code>.
	 */
	public abstract Filter duplicate();
}
	
	