/*	FilterEngine.java		(ProFil)
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil.filter;

import profil.http.Packet;
import java.util.Vector;
import java.util.Enumeration;
import java.awt.Dialog;
import java.awt.List;
import java.io.*;


/**
 * <p>
 * Stores a set of filters and applies them in a right order to a <code>Packet</code>.
 * Each time a new connection is handled (and a new <code>ClientToServerHandler</code>
 * started), an instance of <code>FilterEngine</code> is created which contains
 * 2 lists of clones of the filters belonging to the <code>SiteFilter</code> that
 * matched this connection. The first list contains the filters that should be applied
 * on packets going from a client to a server, and the second contains filters that
 * should be applied on packets going from a server to a client. One filter can be in both
 * lists. Inside these lists, the filters are arranged in order to be executed
 * in a right order when applied on a packet. This arrangement is made by
 * <code>Filtermanager</code>.
 * <p>
 * The filters stored in both lists are cloned obtained through the <code>duplicate()</code>
 * method of the <code>Filter</code>s, and thus allow each filter for each connection
 * to have its own specific variables adapted to this specific connection.
 * <p>
 * <code>FilterEngine</code> provides a method <code>filter()</code> that
 * runs the filters on a packet in a specific filter-dependent order.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public class FilterEngine extends FilterManager
{
	private Vector ctos_filters;
	private Vector stoc_filters;
	
	
	/**
	 * Create a new FilterEngine() containing no filters.
	 */
	public FilterEngine()
	{	stoc_filters 	= new Vector(NUMBER_OF_FILTERS);
		ctos_filters 	= new Vector(NUMBER_OF_FILTERS);
	}
	
	
	/**
	 * Filters a <code>Packet</code>. <code>Filter</code>s are applied to this
	 * <code>Packet</code> in a well defined filter and packet dependent order,
	 * as described in <code>FilterManager</code>.
	 * <p>
	 * @param <code>packet</code> a <code>Packet</code> to filter.
	 */
	public void filter(Packet packet)
	{	
		Vector v;
		if (packet.fromClient())
			v = ctos_filters;
		else
			v = stoc_filters;
			
		for(Enumeration enum = v.elements(); enum.hasMoreElements();)
		{	((Filter)enum.nextElement()).filter(packet);		}
	}
	
	
	/**
	 * Add a new filter to this <code>FilterEngine</code>. If this filter
	 * is already in the <code>FilterEngine</code>, the old one is replaced by the
	 * new one.
	 * <p>
	 * @param <code>filter</code> the filter to add.
	 */	
	public void addFilter(Filter filter)
	{
		Filter f = filter.duplicate();
		int pos;
		int type = f.getTYPE();
		
		pos = searchFilter(type, ctos_filters);
		if (pos == -1)
			insertFilter(f, ctos_filters, true);
		else
			ctos_filters.setElementAt(f, pos);
		
		pos = searchFilter(type, stoc_filters);
		if (pos == -1)
			insertFilter(f, stoc_filters, false);
		else
			stoc_filters.setElementAt(f, pos);
	}
	
	
	/**
	 * Remove this filter from the <code>FilterEngine</code>.
	 * <p>
	 * @param <code>name</code> the name of the filter to remove.
	 */
	public void removeFilter(String name)
	{	int pos;
		int type = nameToType(name);
		
		pos = searchFilter(type, ctos_filters);
		if (pos != -1)
			ctos_filters.removeElementAt(pos);
		
		pos = searchFilter(type, stoc_filters);
		if (pos != -1)
			stoc_filters.removeElementAt(pos);
	}

/*	private void disp()	//disp content of ctos & stoc_filters
	{
		System.out.println("FilterEngine>");
		System.out.println("> ctos filters.size: "+ctos_filters.size());
		
		for (int i=0; i<ctos_filters.size(); i++)
		{	System.out.println("> "+ typeToName( ((Filter)ctos_filters.elementAt(i)).getTYPE()  ) );	}
		
		System.out.println("> stoc filters.size: "+stoc_filters.size());
		for (int i=0; i<stoc_filters.size(); i++)
		{	System.out.println("> "+ typeToName( ((Filter)stoc_filters.elementAt(i)).getTYPE()  ) );	}

		
	}
*/
		

	/**
	 * Required for serialization.
	 */
	protected void load(ObjectInputStream in)  throws IOException, ClassNotFoundException
	{	ctos_filters = (Vector)in.readObject();
		stoc_filters = (Vector)in.readObject();
	}
	
	/**
	 * Required for serialization.
	 */
	protected void save(ObjectOutputStream out)  throws IOException
	{	out.writeObject(ctos_filters);
		out.writeObject(stoc_filters);
	}
	
	
	
	/**
	 * Fill an awt.List with the names of the filters contained in this
	 * <code>SiteFilter</code>. 
	 * <p>
	 * @param <code>l</code> a java.awt.List.
	 */
	public void fillList(List l)
	{
		l.removeAll();
		int type;
		for(Enumeration enum = ctos_filters.elements(); enum.hasMoreElements();)
		{	type = ((Filter)enum.nextElement()).getTYPE();
			l.add(typeToName(type));
		}
		
		Filter f;
		for(Enumeration enum = stoc_filters.elements(); enum.hasMoreElements();)
		{	f = (Filter)enum.nextElement();
			if (!ctos_filters.contains(f))
			{	l.add(typeToName(f.getTYPE()));	}
		}
	}
	
	
	/**
	 * Edit a <code>Filter</code>. Calls the <code>edit()</code>
	 * method of the <code>Filter</code> having this name. If a GUI
	 * is showned, its parent <code>Dialog</code> is <code>owner</code>.
	 * This method crashes if <code>FilterEngine</code> does not contain a
	 * <code>Filter</code> with this <code>name</code>.
	 * <p>
	 * @param <code>name</code> the <code>Filter</code>'s name.
	 * @param <code>owner</code> the parent <code>Dialog</code> of the GUI displayed.
	 */
	public void edit(String name, Dialog owner)
	{
		int pos = searchFilter(nameToType(name), ctos_filters);
		if (pos != -1)
		{	Filter f = (Filter)ctos_filters.get(pos);
			f.edit(owner);
		} else
		{	pos = searchFilter(nameToType(name), stoc_filters);
			Filter f = (Filter)stoc_filters.get(pos);
			f.edit(owner);
		}
	}
	
	
	
	
	
	/*********** PRIVATE ************/
	//serach in 'filters' the filter of type 'type', and return its index
	private int searchFilter(int type, Vector filters)
	{	int pos;
		for (	pos=0;
				pos<filters.size() && ((Filter)filters.elementAt(pos)).getTYPE()!=type;
				pos++	)	{}
		
		if (pos == filters.size())	pos = -1;
		
		return pos;
	}
	
}