/*	FilterManager.java		(ProFil)
 *	Set indentation=4
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/


package profil.filter;

import java.awt.*;
import java.util.Vector;
import java.util.Enumeration;


/**
 * <p>
 * Creates new instances of ProFil's filters, and manages information about the
 * available filters.
 * <p>
 * In ProFil, a filter is a subclass of the abstract class <code>Filter</code>,
 * and each of these subclasses possesses a unique 'identity code' (or <code>TYPE</code>)
 * that can be obtained by its method <code>getTYPE()</code>. Each of these subclasses
 * also possess a unique name (such as <code>"Picture Filter"</code>).
 * <code>FilterManager</code> is in charge of making the connection between the
 * filters' names and <code>TYPE</code>s.
 * <p>
 * For graphical interface convenience, it also provides a method to fill an
 * awt.List with the names of all the filters available.
 * <p>
 * Last but not least, <code>FilterManager</code> knows in which order filters
 * should be applied to a <code>Packet</code>, depending on which filters are applied
 * and of wether this <code>Packet</code> is going from a client to a server or on the
 * way back.
 * <p>
 * If you want to add a new filter to ProFil, what you mainly have to do is to make the
 * main class of this filter extend <code>profil.filter.Filter</code> by overriding
 * its methods <code>getTYPE(), edit(), duplicate()</code> and <code>filter()</code>.
 * This class should be in the package <code>profil.filter</code>. Then you have to update
 * <code>FilterManager</code> so that it contains the new filter. For this, you'll have to edit
 * its source code and add an entry in the <code>private String[] names</code> with
 * the name of your new filter, as it should be displayed in the "Edit SiteFilter"'s GUI.
 * Then, the <code>TYPE</code> of the new filter is simply the index of its name in the previous
 * array. After, you need to add a case in <code>createFilter()</code> so that when this
 * method is called with in parameter the name of the new filter, it returns an instance
 * of the new filter. The last step, is to update the 2 arrays <code>ctos_order</code>
 * and <code>stoc_order</code>, to define the priorities between filters.
 * <p>
 * <code>ctos_order</code> contains an ordered list of type numbers representing the filters
 * that should be applied on a packet going from a client to a server. The <code>Filter</code>
 * whose <code>TYPE</code> is the first in the array is the one to apply at first, the second
 * to apply in second, and so on.
 * <code>stoc_order</code> is the same but for packets going from server to client.
 * If a <code>Filter</code> does not need to be applied on a <code>Packet</code> going
 * in one direction, you just have to NOT put its <code>TYPE</code> in the corresponding
 * array. <code>insertFilter()</code> is in charge of using these 2 arrays to order the 
 * <code>Vector</code> of filters used by <code>FilterEngine</code>s.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */



public class FilterManager
{
	//the array linking filters' names with filters' TYPEs
							//  "type",	"name"
	private final String[] names = {	"0",	"Cookie Filter",
								"1",	"Picture Filter",
								"2",	"Applet Filter",
								//"3",	"Script Filter",
								"4",	"HTTPHeader Filter",
								"5",	"CGI Filter",
								//"6",	"ProxyChainer Anonymiser",						
								//"7",	"Tunneling",
								//"8",	"Offline Browsing",
								//"9",	"HTML Source Browser"
								};
	
	private final int[] ctos_order =	{	0, 1, 4, 5, 8, 9, 6, 7		};
	private final int[] stoc_order =	{	7, 6, 0, 2, 3, 4, 5, 8, 9	};
	
	/**
	 * The number of available filters.
	 */
	protected final int NUMBER_OF_FILTERS = names.length;
	
	
	/**
	 * Create a new instance of the filter defined by <code>name</code>.
	 * The name given in parameter is the String that appeared in
	 * the "Edit SiteFilter"'s GUI for the corresponding filter.
	 * <code>createFilter()</code> then identifies the <code>TYPE</code> number
	 * associated with this filter, and creates an instance of this specific filter.
	 * <p>
	 * @param <code>name</code> the name of the filter to create.
	 * @return a new instance of this filter.
	 */
	 
	public Filter createFilter(String name)
	{	int type = nameToType(name);
		
		switch (type)
		{
			case 0:		return new CookieFilter();
			case 1:		return new PictureFilter();
			case 2:		return new AppletFilter();
		//	case 3:		return new ScriptFilter();
			case 4:		return new HeaderFilter();
			case 5:		return new CGIFilter();
		//	case 6:		return new ProxyChainerFilter();
		//	case 7:		return new TunnelFilter();
		//	case 8:		return new SiteLogger();
			
///////////////////////////////////////////////////////////////		
//to add a filter, just add a line formated as:
//
//			case TYPE:	return new NewFilterConstructor();
//
///////////////////////////////////////////////////////////////
			default:	return new CookieFilter();
		}
	}
	
	
	
	
	/**
	 * Insert a <code>Filter a</code> in a <code>Vector</code> of
	 * <code>Filter</code>s, in a specific order. The <code>Vector</code> of
	 * <code>Filter</code> contains <code>Filter</code>s that should be applied
	 * on <code>Packet</code>s going in the direction specified by the boolean
	 * <code>ctos</code>, and these <code>Filter</code>s should already be ordered
	 * accordingly to the precedence rules for <code>Filter</code>s.
	 * <code>filter</code> will be inserted in the <code>Vector</code>
	 * in order to respect this precedence rules.
	 * <p>
	 * @param <code>filter</code> a <code>Filter</code>.
	 * @param <code>filters</code> a <code>Vector</code> of <code>Filter</code>s,
	 * eventually empty.
	 * @param <code>ctos</code> <code>true</code> if packet is going from client to server,
	 * <code>false</code> otherwise.
	 */
	public void insertFilter(Filter filter, Vector filters, boolean ctos)
	{	
		int[] order;
		if (ctos)	order = ctos_order;
		else		order = stoc_order;

		int pos_a = indexOf(filter.getTYPE(), order);
		
		//if filter a not used in this way (CtoS or StoC), don't put it in filters
		if (pos_a == -1)	return;
		
		//if filters empty, just add filter a
		if (filters.size()==0)
		{	filters.add(filter);
			return;
		}
		
		//otherwise, put filter a in the right position in filters
		Filter b;
		int pos_b;
		
		for (int i=0; i<filters.size(); i++)
		{
			b = (Filter)filters.elementAt(i);
			pos_b = indexOf(b.getTYPE(), order);
		
			if (pos_b > pos_a)
			{	filters.add(i, filter);
				return;
			}
		}
		
		filters.add(filter);
	}
	
	
	
	/* Search an array of integers for a particular integer value.
	 * Returns the position of this integer, or -1 if not
	 * found in the array.
	 */
	private int indexOf(int value, int[] array)
	{
		for(int i=0; i<array.length; i++)
		{	if (array[i]==value)	return i;	}
		return -1;
	}

	
	
	/**
	 * Returns the <code>TYPE</code> of the filter having this <code>name</code>.
	 * <p>
	 * @param <code>name</code> a filter's <code>name</code> (see <code>createFilter()</code> for details).
	 * @return the filter's <code>TYPE</code>.
	 */
	
	public int nameToType(String name)
	{	
		int pos;
		for(	pos=1;
				pos<NUMBER_OF_FILTERS && !names[pos].equals(name);
				pos+=2
			)	{}
	
		if (pos >= NUMBER_OF_FILTERS)	return -1;
		
		try {	int type = Integer.parseInt(names[pos-1]);
				return type;
		} catch (Exception e) {}
		return -1;	//no use: just fool javac	
	}
	
	
	/**
	 * Returns the <code>name</code> of the filter having this <code>TYPE</code>.
	 * <p>
	 * @param <code>type</code> a filter's <code>TYPE</code>.
	 * @return the filter's <code>name</code>.
	 */
	 
	public String typeToName(int type)
	{	int pos;
		String s_type = ""+type;
		try {
				for(	pos=0;
						pos<NUMBER_OF_FILTERS && !names[pos].equals(s_type);
						pos+=2
					)	{}
	
				if (pos >= NUMBER_OF_FILTERS)	return "type not defined";
		
				return names[pos+1];
		} catch (Exception e) {}
		return "";	//no use: just fool javac	
	}
	
	
	/**
	 * Fill a List with the <code>name</code>s of all available filters.
	 * <p>
	 * @param <code>l</code> a java.awt.List.
	 */
	 
	public void listAllFilters(List l)
	{	for(int i=1; i<NUMBER_OF_FILTERS; i+=2)
		{	l.add(names[i]);	}
	}
	
}
