/*	HTTPHeaders.java		(ProFil)
 *	Set indentation=4
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil.http;

import java.util.Vector;

/**
 * <p>
 * When an HTTP request or reply is received by ProFil, its HTTP headers are
 * stored in an instance of class <code>HTTPHeaders</code>.
 * Thus, <code>HTTPHeaders</code> basicaly contains a list of the headers
 * received, and a list of their corresponding values. <code>HTTPHeaders</code>
 * provides methods to add a new header, remove a header, obtain a header's value,
 * or build a string representing these headers.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public class HTTPHeaders
{
	private Vector headers;
	private Vector values;
	private Vector lc_headers;
	
	//code for new_line in HTTP
	private static String CRLF = "\r\n";
	
	
	/**
	 * Creates a new empty <code>HTTPHeaders</code>.
	 */
	public HTTPHeaders()
	{	
		this.headers 	= new Vector(10);
		this.values  	= new Vector(10);
		this.lc_headers = new Vector(10);
	}
	
	
	/**
	 * Adds a header to this <code>HTTPHeaders</code>. The String <code>line</code>
	 * must contain the header's name and value as it would be used in an HTTP
	 * request or reply. For exemple, to add a header <code>Set-Cookie</code> with
	 * value <code>name=Bill</code> to the <code>HTTPHeaders</code>, <code>line</code>
	 * must be formated as follows: <code>"Set-Header: name=Bill"</code>.
	 * <p>
	 * @param <code>line</code> a String formated as <code>"Header-Name: header_value"</code>.
	 */
	public void addHeader(String line)
	{	
		int pos = line.indexOf(":");
		if (pos == -1)	return;			//invalid header
	
		String h = line.substring(0, pos);
		headers.add(h);
		lc_headers.add(h.toLowerCase());
		values.add( (line.substring(pos+1)).trim() );
	}
	
	
	/**
	 * A String representation of the headers stored in this <code>HTTPHeaders</code>.
	 * <code>toString</code> enumerates the headers in this <code>HTTPHeaders</code> and builds
	 * a String with as many lines as there are headers, and in which each line is formated
	 * as <code>"Header-Name: header_value"</code>. The <code>new_line</code> characters
	 * separating 2 lines are CR and LF, according to the HTTP protocol. Thus this String can
	 * be directly written on an HTTP socket's <code>OutputStream</code>.
	 * <p>
	 * @return the String representing these headers, formated to be written on an HTTP stream.
	 */
	public String toString()
	{
		String h = "";
		for (int i =0; i<headers.size(); i++)
		{	h += (String)headers.elementAt(i);
			h += ": ";
			h += (String)values.elementAt(i);
			h += CRLF;
		}
		return h;
	}
	
	
	/**
	 * Removes the specified <code>header</code> from this <code>HTTPHeaders</code>. The String
	 * <code>header</code> is the name of the HTTP header to remove (for example
	 * <code>"From"</code> or <code>"Set-Cookie"</code>). If <code>header</code>
	 * is not present in <code>HTTPHeaders</code>, nothing is done.
	 * <p>
	 * @param <code>header</code> the name of the header to remove.
	 */
	public void removeHeader(String header)
	{
		header = header.toLowerCase();
		int p = lc_headers.indexOf(header);
		if (p==-1)	return;
		headers.removeElementAt(p);
		values.removeElementAt(p);
		lc_headers.removeElementAt(p);
	}
	
	
	/**
	 * Returns the value associated with <code>header</code>. If <code>header</code>
	 * is not present in this <code>HTTPHeaders</code>, an empty String <code>""</code>
	 * is returned.
	 * <p>
	 * @param <code>header</code> a header's name, like <code>"Set-Cookie"</code>.
	 * @return the value associated with this header, or <code>""</code>
	 * if there is no such header.
	 */
	public String getHeaderValue(String header)
	{
		int pos = lc_headers.indexOf(header.toLowerCase());
		if (pos==-1)	return "";
		
		return (String)values.elementAt(pos);
	}
	
	
	
	/**
	 * Set the <code>value</code> associated <code>header</code>. If <code>header</code>
	 * is not present in this <code>HTTPHeaders</code>, nothing is done.
	 * <p>
	 * @param <code>header</code> a header's name, like <code>"Set-Cookie"</code>.
	 * @param <code>header</code> the avlue to which the header has to be set.
	 */
	public void setHeaderValue(String header, String value)
	{
		int pos = lc_headers.indexOf(header.toLowerCase());
		if (pos != -1)
		{
			values.setElementAt(value, pos);
		}
	}
}
		