/*	HTTPReply.java		(ProFil)
 *	Set indentation=4
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil.http;


/**
 * <p>
 * When ProFil receives a server's reply to a previous request, it stores the
 * first line of the reply in an instance of class <code>HTTPReply</code>.
 * This first line contains the version of HTTP used by the server, the reply's
 * code for this request (like 404 for 'document not found' or 200 for 'ok'), and
 * a ends with a text line commenting the code.
 * <p>
 * <code>HTTPReply</code> provides methods to parse this firs line, and to get or
 * modify each of its fields, as to build a String representing the reply.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */


public class HTTPReply
{
	private static String CRLF = "\r\n";
	
	private String http;
	private String code;
	private String text;
	
	
	/**
	 * Builds a new <code>HTTPReply</code>. <code>line</code> must be the first line
	 * received from the server, wihtout the CRLF last characters. It is typicaly formated
	 * as <code>http_version, reply_code, reply_text</code>.
	 * <p>
	 * @param <code>line</code> the line sent by a HTTP server giving its answer.
	 */
	public HTTPReply(String line)
	{	int pos = line.indexOf(" ");
		http = line.substring(0, pos);
		line = (line.substring(pos+1)).trim();
		
		pos = line.indexOf(" ");
		code = line.substring(0, pos);
		
		text = line.substring(pos+1);
	}
	
	
	/**
	 * Gives a String representation of a Reply's first line. The returned
	 * String is formated as <code>http_version reply_code reply_text CRLF</code>, 
	 * and thus already contains the HTTP <code>new_line</code> characters CRLF.
	 * <p>
	 * @return the reply's first line as a String to write directly in an HTTP Stream.
	 */
	public String toString()
	{	return http+" "+code+" "+text+CRLF;		}
	
	
	/**
	 * Returns the version of HTTP used by the server.
	 * <p>
	 * @return <code>"HTTP/1.0"</code> or <code>"HTTP/1.0"</code>.
	 */
	public String getHttp()			{	return http;	}
	
	
	/**
	 * Returns the reply's code sent by the server.
	 * <p>
	 * @return the reply's code as a String ("200", "404"...).
	 */
	public String getCode()			{	return code;	}
	
	
	/**
	 * Returns the text associated with the reply's code.
	 * <p>
	 * @return the text associated with the reply's code.
	 */
	public String getText()			{	return text;	}
	
	
	/**
	 * Set the value of the HTTP version used in this <code>HTTPReply</code>.
	 * <p>
	 * @param <code>protocol</code> <code>"HTTP/1.0"</code> or <code>"HTTP/1.0"</code>.
	 */
	public void setHttp(String protocol)	{	this.http = protocol;	}
	
	
	/**
	 * Set the reply's code.
	 * <p>
	 * @param <code>code</code> <code>"404"</code>, <code>"200"</code>, etc...
	 */
	public void setCode(String code)	{	this.code = code;	}
	
	
	/**
	 * Set the text associated with the reply's code.
	 * <p>
	 * @param <code>text</code> <code>"Document not found"</code>, etc...
	 */
	public void setText(String text)	{	this.text = text;	}
}
