/*	HTTPRequest.java		(ProFil)
 *	Set indentation=4
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/


package profil.http;

/**
 * <p>
 * When ProFil receives a request to transmit, it stores the first line of this request
 * in an instance of class <code>HTTPRequest</code>. This first line contains the method
 * used for the request (GET, POST or HEAD), its protocol, server address, document's URI,
 * and version of HTTP used.
 * <code>HTTPRequest</code> provides access to all these fields.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */

public class HTTPRequest
{
	private Proxy proxy;
	
	private String method 	= "";
	private String protocol = "";
	private String server	= "";
	private String uri		= "";
	private String http		= "";

	private static String CRLF = "\r\n";
	
	
	/**
	 * Create a new <code>HTTPRequest</code>. <code>line</code> is the first line of
	 * request received on a Socket <code>InputStream</code> without CRLF. <code>proxy</code>
	 * is the proxy used by ProFil. It is required by the method <code>toString</code> which
	 * will modify the server's address depending on if ProFil use an intermediary proxy or
	 * not.
	 * <p>
	 * @param <code>line</code> The first line in an HTTP request, without CRLF et the end.
	 * @param <code>proxy</code> The <code>proxy</code> used by ProFil.
	 */
	public HTTPRequest(String line, Proxy proxy)
	{
		this.proxy = proxy;
		
		int pos1, pos2;
		
		line.trim();
		pos1 = line.indexOf(" ");
		method = line.substring(0, pos1);
		line = (line.substring(1+pos1)).trim();
		
		pos1 = line.indexOf("://");
		if (pos1 != -1)
		{
			pos1 += 3;
			protocol = line.substring(0, pos1);
			pos2 = line.indexOf("/", pos1);
			server = line.substring(pos1, pos2);
			line = line.substring(pos2);
		}
		
		pos1 = line.indexOf(" ");
		uri = line.substring(0, pos1);
		http = (line.substring(pos1)).trim();
	}
	
	
	/**
	 * Gives a String representing the HTTP first line of request represented by this
	 * <code>HTTPRequest</code>. The String ends with a CRLF and is formated in order to
	 * be directly sent on an HTTP <code>OutputStream</code>.
	 */
	public String toString()
	{	
		String s = method+" ";
		
		if (proxy.isEnabled())
			s += protocol+server;
		s += uri+" "+http+CRLF;
		
		return s;
	}
	
	
	/**
	 * Returns the request's method. Typicaly <code>"GET"</code>, <code>"POST"</code>
	 * or <code>"HEAD"</code>.
	 * <p>
	 * @return the request's method.
	 */
	public String getMethod()		{	return method;		}
	
	
	/**
	 * Returns the request's protocol. Typicaly <code>"http://"</code>, <code>"ftp://"</code>
	 * or <code>"mailto://"</code>. Returns <code>""</code> if there was no protocol in the request.
	 * <p>
	 * @return the request's protocol.
	 */
	public String getProtocol()		{	return protocol;	}
	
	
	/**
	 * Returns the request's target server. Formated as
	 * <code>"address:port"</code> or <code>"address"</code> if <code>port</code> = 80.
	 * <p>
	 * @return the request's target server.
	 */
	public String getServer()		{	return server;		}
	

	/**
	 * Returns the URI of the requested document.
	 * <p>
	 * @return the URI of the requested document.
	 */	
	public String getURI()			{	return uri;			}
	
	
	/**
	 * Returns the version of HTTP recognized by the client.
	 * <p>
	 * @return the version of HTTP.
	 */	
	public String getHttp()			{	return http;		}
	
	
	public void setMethod(String s)		{	this.method 	= s;	}
	public void setProtocol(String s)	{	this.protocol	= s;	}
	public void setServer(String s)		{	this.server 	= s;	}
	public void setURI(String s)		{	this.uri		= s;	}
	public void setHttp(String s)		{	this.http		= s;	}
}