/*	Packet.java		(ProFil)
 *	Set indentation=4
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/

package profil.http;

import java.util.StringTokenizer;

/**
 * <p>
 * The object used by ProFil to cache and analyse data going through a client-server connection.
 * <p>
 * For each connection that ProFil creates between a client and a server, ProFil creates
 * 2 instances of class <code>Packet</code>, one to cache and analyse the data stream going
 * from the client to the server, a second to handle the data stream from the server to the
 * client.
 * <p>
 * A <code>Packet</code> is in charge of caching the data received by ProFil from the server
 * or the client, and to analyse them in order to identify requests, replies, headers and contents.
 * It is also designed to be manipulated by the filters. Each connection between a client
 * and a server is filtered by a group of <code>Filter</code> through a <code>FilterEngine</code>
 * itself built by the <code>SiteFilterStore</code>. Each of the <code>Filter</code> contains
 * in the connection's <code>FilterEngine</code> are applied in turn to both <code>Packet</code>s
 * defined above. Thus, <code>Packet</code> must provides methods to access the various elements
 * that composed an HTTP stream: <code>HTTPRequest</code>, <code>HTTPReply</code>,
 * <code>HTTPHeaders</code> or content.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */





public class Packet
{
	private static String CRLF 	= "\r\n";
	private static String CR 	= "\r";
	private static String LF 	= "\n";
	
	private String input		= "";
	private String content		= "";
	
	private HTTPRequest request;
	private HTTPReply reply;
	private HTTPHeaders headers;

	//boolean used for internal synchronization
	private boolean ctos;								//ctos true if client to server's way, fase if opposite way
	private boolean wait_header			= true;			//true = waiting header
	private boolean wait_content		= false;		//true = content expected
	private boolean header_available	= false;		//true if there are some headers to retransmit
	private boolean content_available	= false;		//true if there is some content to retransmit
	private boolean send_allowed		= true;
	private int content_length;
	private Proxy proxy;
	
		
		
	/**
	 * Create a <code>Packet</code> to be used by a <code>ClientToServerHandler</code>.
	 * The <code>proxy</code> is required to be provided to the <code>HTTPRequest</code>
	 * of this <code>Packet</code>.
	 * <p>
	 * @param proxy the <code>proxy</code> used by ProFil.
	 */
	public Packet(Proxy proxy)
	{	this.ctos = true;
		this.proxy = proxy;
	}
	
	
	/**
	 * Create a <code>Packet</code> to be used by a <code>ServerToClientHandler</code>.
	 */
	public Packet()
	{	this.ctos = false;	}
	
	
	
	
	/**
	 * Add a line of input to this <code>Packet</code>. The line is parsed depending on what
	 * the <code>Packet</code> has already received for request, content, reply or headers.
	 * The String <code>line</code> must be directly taken from an HTTP <code>InputStream</code>,
	 * and thus still contain CRLF as a line separator.
	 * <p>
	 * @param new_input a String taken from an HTTP <code>InputStream</code>.
	 */
	public void add(String new_input)
	{	
		input += new_input;

		if (wait_header)	analyseHeader();
		if (wait_content)	analyseContent();
	}
	
	
	//if an HTTP request/reply is completly received (request/reply line+all headers)
	//parse it and build its corresponding HTTPRequest/Reply and HTTPHeaders
	private void analyseHeader()
	{	
		int pos, p1, p2, p3;
		String separator;
		p1 = input.indexOf(CRLF+CRLF);
		p2 = input.indexOf(CR+CR);
		p3 = input.indexOf(LF+LF);

		if (p1 != -1)
		{	separator = CRLF;
			pos = p1;
		} else if (p2 != -1)
		{	separator = CR;
			pos = p2;
		} else if (p3 != -1)
		{	separator = LF;
			pos = p3;
		} else
			return;

		header_available = true;
		String header = input.substring(0, pos);
	 	input = input.substring( pos+2*(separator.length()) );
		
		StringTokenizer tokens = new StringTokenizer(header, separator);
		if (ctos)	request = new HTTPRequest(tokens.nextToken(), proxy);
		else		reply   = new HTTPReply(tokens.nextToken());

		headers = new HTTPHeaders();
		while (tokens.hasMoreElements())
		{	headers.addHeader(tokens.nextToken());		}
	
		checkIfContent();
	}
		

	
	
	//parse the content and update content_length
	private void analyseContent()
	{		
		int size = input.length();
		if (size == 0)	return;

		if (size < content_length)
		{
			content_length -= input.length();
			content += input;
			input = "";
		} else
		{
			content += input.substring(0, content_length);
			input = input.substring(content_length);
			content_length = 0;
			wait_header = true;
			wait_content = false;
		}
		
		content_available = true;
	}
	
	
	//try to estimate how much bytes of content are expected for this request/reply message
	//messy part, not fully implemented... complain to RFC 2068 ! ;}
	private void checkIfContent()
	{	
		if (ctos)	//in case client to server stream, only a POST query can possess an entity
		{
			if (request.getMethod().equals("POST"))
			{
				content_length = getContentLength();		//POST => there is a content-length
				
				wait_header = false;
				wait_content = true;
			}
			else
			{	
				wait_header = true;
				wait_content = false;
			}
		}
		else	//server to client case: much more complex...
		{
			//first, check if the reply's code implies that there is no content
			String code = reply.getCode();
			if (code.equals("204") || code.equals("304") || code.substring(0,1).equals("1"))
			{	wait_header = true;
				wait_content = false;
				return;
			}
			
			//then, check for content-length
			content_length = getContentLength();
			if (content_length != -1)
			{	wait_header = false;
				wait_content = true;
				return;
			}
			
		/*	if ( headers.getHeaderValue("content-type").length() != 0 )
			{
				wait_content = true;
				wait_header = false;
				content_length = 1000000;
			}
		*/

			//default
			wait_content = true;
			wait_header = false;
			content_length = 1000000;
		}
	}
	
	//extract the content-length header of an HTTPReply/Request
	private int getContentLength()
	{
		String cl = headers.getHeaderValue("content-length");
		
		if (cl.length() == 0)	return -1;
		
		int l = 0;
		try {	l = Integer.parseInt(cl);	}
		catch (Exception e)	{}
		
		return l;
	}
	
	/**
	 * Return the String of data to write on an HTTP <code>OutputStream</code>.
	 * This data is the same data as received by <code>Packet</code> through its
	 * method <code>add()</code> but after having being cached, parsed, eventually filtered
	 * and eventually modify to handle an intermediary proxy.
	 * <p>
	 * @return the data to write on an HTTP <code>OutputStream</code>.
	 */
	public String get()
	{
		String s = "";
		if (header_available)
		{			
			if (ctos)	s += request.toString();
			else		s += reply.toString();
			
			s += headers.toString()+CRLF;
			header_available = false;
		}
		
		if (content_available)
		{
			s += content;
			content = "";
			content_available = false;
		}

		return s;
	}
	
	/**
	 * Test if this <code>Packet</code> has fully received an HTTP header.
	 * If it is the case, it means an HTTPRequest or HTTPReply and an
	 * HTTPHeaders are available for analysing and filtering.
	 * <p>
	 * @return <code>true</code> if yes, <code>false</code> otherwise.
	 */
	public boolean 	headerReceived()		{	return header_available;		}
	
	
	/**
	 * Test wether this <code>Packet</code> is waiting for content or not.
	 * <p>
	 * @return <code>true</code> if yes, <code>false</code> otherwise.
	 */
	public boolean 	waitingContent()		{	return wait_content;			}
	
	
	/**
	 * Test wether this <code>Packet</code> is allowed to be sent.
	 * When a Request <code>Packet</code> cannot be sent (because of some
	 * filter interdiction), the <code>ClientToServerHandler</code> directly
	 * sends back to the server a reply with code 404 (not found).
	 * <p>
	 * @return <code>true</code> if yes, <code>false</code> otherwise.
	 */
	public boolean 	canBeSent()				{	return send_allowed;			}
	
	
	/**
	 * Forbid a <code>Packet</code> to be sent.
	 */
	public void 	doNotSend()				{	send_allowed = false;			}
	
	
	/**
	 * Test wether this <code>Packet</code> handles data from client to server,
	 * or the other way round.
	 * <p>
	 * @return <code>true</code> if it is a client to server's packet,
	 * <code>false</code> otherwise.
	 */
	public boolean 	fromClient()			{	return ctos;					}
	
	
	/**
	 * Returns the content received by this <code>Packet</code>. It is typicaly
	 * used with method <code>setContent()</code> in order to retrieve the content,
	 * filter or modify it, then set it back to be sent.
	 * <p>
	 * @return the content received by this <code>Packet</code>.
	 */
	public String	getContent()			{	return content;		}
	
	
	/**
	 * Set this <code>Packet</code>'s content to this String. 
	 * <p>
	 * @return
	 */
	public void 	setContent(String s)	{	this.content = s;	}
	
	
	/**
	 * The <code>Packet</code>'s HTTPRequest, if available. A call to
	 * <code>headerReceived()</code> has to be made first.
	 * <p>
	 * @return the <code>Packet</code>'s HTTPRequest.
	 */
	public HTTPRequest 	getRequest()		{	return request;		}
	
	
	/**
	 * The <code>Packet</code>'s HTTPReply, if available. A call to
	 * <code>headerReceived()</code> has to be made first.
	 * <p>
	 * @return the <code>Packet</code>'s HTTPReply.
	 */
	public HTTPReply 	getReply()			{	return reply;		}
	
	
	/**
	 * The <code>Packet</code>'s HTTPHeaders, if available. A call to
	 * <code>headerReceived()</code> has to be made first.
	 * <p>
	 * @return the <code>Packet</code>'s HTTPHeaders.
	 */
	public HTTPHeaders 	getHeaders()		{	return headers;		}
}
