/*	Proxy.java		(ProFil)
 *	Set indentation=4
 *
 *	Erwan Lemonnier - september 2000
 *
 *****************************************/


package profil.http;

import java.io.*;
import java.net.*;
import java.io.Serializable;

/**
 * <p>ProFil can be configured to use an intermediary proxy through which to transmit the requests it receives.
 * This can be usefull especially when using ProFil in a coorporate environment where a proxy separates the
 * local intranet from the rest of the internet. This intermediary proxy is described
 * through an object of class <code>Proxy</code>.<p>
 * Thus, <code>Proxy</code> contains all the attributes enabling the use of an intermediary
 * proxy (address, port, enable flag) and the methods to access these attributes, or to connect to the proxy.
 * <p>
 * @author 		Erwan Lemonnier <code><a href="mailto:erwan_lemonnier@hotmail.com">[erwan_lemonnier@hotmail.com]</a></code>
 * @version 	<code>1.1 [released 09/2000]</code>
 */

public class Proxy implements Serializable
{
	private String address;
	private int port;
	private boolean enable;
	
	
	/**
	 * Creates a new instance of <code>Proxy</code>. The proxy represented by <code>Proxy</code>
	 * has the address <code>address</code>, runs on port <code>port</code> and is enabled if
	 * <code>state = true</code>, disabled otherwise.
	 *
	 * @param <code>address</code> the address of the proxy, for exemple "www.proxy.net", or "192.168.1.1".
	 * @param <code>port</code> the port number on which this proxy runs (typically 3128, 80, 8080...).
	 * @param <code>state</code> <code>true</code> is ProFil will use this proxy, <code>false</code> if not.
	 */
	public Proxy(String address, int port, boolean state)
	{	this.address	= address;
		this.port		= port;
		this.enable		= state;
	}
	
	
	/**
	 * Returns a String representation of this proxy. It is formated
	 * as <code>"address:port"</code>. If <code>port = 0</code>, it returns <code>""</code>.
	 * <p>
	 * @return the String <code>"address:port"</code> if the proxy is well defined, "" otherwise.
	 */
	public String toString()
	{	if (port == 0)
			return "";
		else
			return address+":"+port;
	}
	
	
	/**
	 * Tests wether the proxy is enabled or not. If <code>Proxy</code> is enabled, ProFil will
	 * transmit all the requests it receives to their target server via this proxy, otherwise,
	 * the requests are directly transmitted to the server.
	 * <p>
	 * @return <code>true</code> if ProFil will use the proxy, <code>false</code> otherwise.
	 */
	public boolean isEnabled()
	{	return enable;	}


	/**
	 * Returns a socket to the proxy or http server to which ProFil will connect.
	 * Given a String representing the <code>address:port</code> of a request's
	 * destination server, it returns a <code>Socket</code> to connect to it. If
	 * a proxy is enable, this <code>Socket</code> is connected to this proxy,
	 * otherwise it is connected directly to the server.
	 * <p>
	 * @param server <code>"address:port"</code> of request's destination server
	 * @return a <code>Socket</code> to wether the intermediary proxy or the server itself
	 */
	public Socket getServerSocket(String server) throws Exception
	{
		if ( enable && (port != 0) )
		{
			return new Socket(address, port);
		}
		else
		{
			int pos, host_port = 80;
			String host_address = server;
		
			pos = server.indexOf(":");
			if (pos != -1)
			{
				host_port = Integer.parseInt(host_address.substring(1+pos));
				host_address = host_address.substring(0, pos);
			}
			return new Socket(host_address, host_port);
		}
	}
}

	